MODULE OrdnerADT EXPORTS OrdnerADT;
(* Implementierung des ADT Ordner, mit durch zusicherungen geschtzten Funktionen.

Autor : Horst Lichter, RWTH Aachen
Umgebung : SRC-Modula-3 rel. 3.6, Windows NT 4.0
Erstellt : 16.08.98
Letzte Aenderung: 09.01.2001
*)

IMPORT Assertion AS As;

CONST MaxTexte = 20;
TYPE Fassungsvermoegen = [1 .. MaxTexte];
TYPE Texte = ARRAY Fassungsvermoegen OF TEXT;

REVEAL Ordner = BRANDED REF RECORD
                  ordnerInhalt : Texte;
                  anzahlTexte : CARDINAL;
                  beschriftung : TEXT := "";
                END;


PROCEDURE LegeTextAb (VAR o: Ordner; t : TEXT)RAISES {As.Violated} =
(* Legt einen Text im Ordner o ab *)
BEGIN
  As.Require (NOT IstVoll(o), "OrdnerADT.LegeTextAb");	(* Vorbedingung *)
  o^.anzahlTexte := o^.anzahlTexte + 1;
  o^.ordnerInhalt[o^.anzahlTexte] := t;
  As.Ensure (NOT IstLeer(o), "OrdnerADT.LegeTextAb");	(* Nachbedingungen *)
  As.Ensure (Invariante(o), "OrdnerADT.LegeTextAb");
END LegeTextAb;

PROCEDURE EntnehmeText (VAR o: Ordner) : TEXT RAISES {As.Violated}=
(* liest den letrzten Text des Ordners o, gibt ihn zurck und lscht ihn dann aus dem Ordner *)
VAR t : TEXT;
BEGIN
  As.Require (NOT IstLeer(o), "OrdnerADT.EntnehmeText");	(* Vorbedingung *)
  t := o^.ordnerInhalt[o^.anzahlTexte];
  o^.ordnerInhalt[o^.anzahlTexte] := "";
  o^.anzahlTexte := o^.anzahlTexte - 1;
  As.Ensure (NOT IstVoll(o), "OrdnerADT.EntnehmeText");		(* Nachbedingungen *)
  As.Ensure (Invariante(o), "OrdnerADT.EntnehmeText");		
  RETURN t;
END EntnehmeText;

PROCEDURE IstVoll (o : Ordner) : BOOLEAN =
(* Test Funktion um vor- und nachbedingungen zu formulieren *)
BEGIN
  RETURN (o^.anzahlTexte = MaxTexte);
END IstVoll;

PROCEDURE IstLeer (o : Ordner) : BOOLEAN =
(* Test Funktion um vor- und nachbedingungen zu formulieren *)
BEGIN
  RETURN (o^.anzahlTexte = 0);
END IstLeer;

PROCEDURE Beschrifte (VAR o: Ordner; t : TEXT) =
(* Legt den Titel des Ordners fest *)
BEGIN
  o^.beschriftung := t;
END Beschrifte;

PROCEDURE GibBeschriftung (o : Ordner) : TEXT =
(* Liest den Titel der Ordners o und gibt diesen zurck *)
BEGIN
  RETURN o^.beschriftung;
END GibBeschriftung;

PROCEDURE Anlegen () : Ordner RAISES {As.Violated} =
(* Erzeugt eine neue Instanz des ADTs Ordner und initalisiert diesen *)
VAR o : Ordner;
BEGIN
  o := NEW(Ordner);
  FOR i := FIRST(Fassungsvermoegen) TO LAST(Fassungsvermoegen) DO
    o^.ordnerInhalt[i] := "";
  END;
  o^.anzahlTexte := 0;
  As.Ensure(IstLeer(o) AND Invariante(o), "Ordner.Anlegen"); (* Nachbedingung *)
  RETURN o;
END Anlegen;

PROCEDURE Invariante (o : Ordner): BOOLEAN =
BEGIN
  RETURN (o^.anzahlTexte >= 0 AND o^.anzahlTexte <= MaxTexte);
END Invariante;

BEGIN
END OrdnerADT.