MODULE Ordner_Test EXPORTS Main;
(* 
Ein Beispiel fr den einsatz von Assertions und die Behandlung von Exceptions.

Autor : Horst Lichter, RWTH Aachen
Umgebung : SRC-Modula-3 rel. 3.6, Windows NT 4.0
Erstellt : 16.08.98
Letzte Aenderung: 11.01.2001
*)

IMPORT Assertion AS As; (* Module werden importiert *)
IMPORT OrdnerADT,  SIO;


PROCEDURE ArbeiteMitOrdner()RAISES {As.Terminate}=	(* Kommt es zu einer Exception wird der fehler Terminate erzeugt *)
(* Hier werden einige Operationen auf dem ADT ausgefhrt *)
VAR ordner1, ordner2 : OrdnerADT.Ordner;	
BEGIN
  TRY	(* Ausnahmen werden abgefangen *)
    ordner1 := OrdnerADT.Anlegen();			(* Eine neue Instanz des ADT wird initialisiert *)
    OrdnerADT.Beschrifte (ordner1, "Kleine Gedichte");	(* Der Ordner wird mit einigen Bsp. Texten gefllt *)
    OrdnerADT.LegeTextAb (ordner1, "Nicht immer sind bequeme Stuehle ...");
    OrdnerADT.LegeTextAb (ordner1, "Herr von Ribeck auf Ribeck ...");
    OrdnerADT.LegeTextAb (ordner1, "Von drauss vom Walde komm ich her ...");

    ordner2 := OrdnerADT.Anlegen();			(* Ein zweiter Ordner wird initialisiert *)
    OrdnerADT.Beschrifte (ordner2, "Musikstuecke");	(* und mit "Texten gefllt *)
    OrdnerADT.LegeTextAb (ordner2, "We all live in a yellow ...");
    OrdnerADT.LegeTextAb (ordner2, "Tief im Westen wo die Sonne verstaubt ...");
    OrdnerADT.LegeTextAb (ordner2, "Drei alte Tanten tanzen Tango mitten in der Nacht ...");


    SIO.PutLine (OrdnerADT.GibBeschriftung(ordner1));	(* Beschriftung wird gelesen *)
    SIO.PutLine ("---------------------------");
    SIO.PutLine (OrdnerADT.EntnehmeText(ordner1)); 	(* Texte werden gelesen *)
    SIO.PutLine (OrdnerADT.EntnehmeText(ordner1));  
    SIO.PutLine (OrdnerADT.EntnehmeText(ordner1));  
    SIO.PutLine (OrdnerADT.EntnehmeText(ordner1));  	(* Hier ist der Ordner bereits leer eine Ausnahme wird generiert *)


	(* Dieser Teil wird garnicht erreicht wegen dem vorherigen Fehler *)
    SIO.PutLine (OrdnerADT.GibBeschriftung(ordner2));	
    SIO.PutLine ("---------------------------");
    SIO.PutLine (OrdnerADT.EntnehmeText(ordner2)); 
    SIO.PutLine (OrdnerADT.EntnehmeText(ordner2));  
    SIO.PutLine (OrdnerADT.EntnehmeText(ordner2));  
    SIO.PutLine (OrdnerADT.EntnehmeText(ordner2));	(* Hier ist der Ordner bereits leer eine Ausnahme wird generiert *)

(* Hier wird die Exception behandelt aus Zeile 36 vgl. OrdnerADT.i3 *)
  EXCEPT  
    As.Violated => SIO.PutLine ("*** Called from: ArbeiteMitOrdner");
                   RAISE As.Terminate;
  END;
END ArbeiteMitOrdner;
  
BEGIN
(* Hauptprogramm *)
  As.EnableAssertions();	(* Zusicherungen sind "eingeschaltet" d.h. Exceptions werden *)
				(* erzeugt wenn eine Bedingung nicht erfllt ist *)

  TRY				(* Falls eine ausnahme auftritt wird sie hier behandelt *)
    ArbeiteMitOrdner();
  EXCEPT			(* Die mglicherweise von von "ArbeiteMitOrdner();" erzeugte ausnahme "As.Terminate *)
				(* wird hier behandelt *)
    As.Terminate => SIO.PutLine ("*** Program terminated due to assertion violation");	(* Falls die ausnahme auftritt wird ...*)
											(* ...eine Fehlermeldung ausgegeben *)
  END
END Ordner_Test.